
# read Universal Dependency 'conllu' files and serialize them as RDS
ReadUD <- function(file, serialize = TRUE, dir = getwd()) {
  # read raw data
    message(paste0("Reading '",
                   gsub("^.+/(.+)/.+\\.conllu$", "\\1", file), "_",
                   gsub("^.+/.+ud-(.+)\\.conllu$", "\\1", file), "'"))
    
    x <- fread(file,
               sep = "",
               header = FALSE,
               stringsAsFactors = FALSE,
               blank.lines.skip = TRUE,
               encoding = "UTF-8")
    
    # split data into columns
    x <- data.table(str_split_fixed(x[!grepl("^#", V1), V1], "\t", 10))
    setnames(x, c("wID", "word", "lemma", "pos", "pos2", "features", "rID", "rType", "rType2", "etc"))
    
    # index sentences
    x[wID == "1", sID := seq(1, .N, 1)]
    x[, sID := sID[which(!is.na(sID))], by = cumsum(!is.na(sID))]
    
    # add information on UD corpus, file, and language
    x[, corpus := gsub("^.+/(.+)/.+\\.conllu$", "\\1", file)]
    x[, file := gsub("^.+/.+ud-(.+)\\.conllu$", "\\1", file)]
    x[, language := gsub("^UD_(.+)-.+$", "\\1", corpus)]
    
    # adjust column order
    setcolorder(x, c("language", "corpus", "file", "sID"))
    
    # serialize or return data
    if (serialize) {
      # serialize data
      saveRDS(x, file = paste0(dir, "/",
                               gsub("^.+/(.+)/.+\\.conllu$", "\\1", file),
                               "_",
                               gsub("^.+/.+ud-(.+)\\.conllu$", "\\1", file),
                               ".rds"),
              ascii = FALSE, compress = TRUE)
    } else {
      # return table
      return(x)
    }
}

